
#ifndef ACCOUNT_H
#define ACCOUNT_H

#include <iostream>

#define IRATEBLOCKED   4
#define IRATEUNBLOCKED 2

class Account {
    private:
        unsigned int m_number;
    protected:
        double m_balance;
    public:
        Account(unsigned int number, double balance);
        virtual ~Account();

        void credit(double amount);
        unsigned int operator()() const;
        
        virtual double debit(double amount) = 0;
        virtual void print(std::ostream& os) const = 0;
        virtual Account* clone() const = 0;
};


/* current account */
class CurrentAccount : public Account {
    public:
        CurrentAccount(unsigned int number, double balance);
        double debit(double amount) override;
        void print(std::ostream& os) const override;
        CurrentAccount* clone() const override;
};


/* saving accounts */
class SavingAccount: public Account {
    private:
        double m_interest_rate;
    public:
        SavingAccount(unsigned int number, double balance, double interest_rate);
        void print(std::ostream& os) const final override;
        double add_interests();
};

class BlockedSavingAccount: public SavingAccount {
    public:
        BlockedSavingAccount(unsigned int number, double balance);
        double debit(double amount) override;
        BlockedSavingAccount* clone() const override;
};

class UnblockedSavingAccount: public SavingAccount {
    public:
        UnblockedSavingAccount(unsigned int number, double balance);
        double debit(double amount) override;
        UnblockedSavingAccount* clone() const override;
};

std::ostream& operator<<(std::ostream& os, const Account& account);

#endif /* end of include guard: ACCOUNT_H */
